// Moebius03
// cerchiamo di fare anche le normali.
// disegnamo semplicemente unendo i punti con una strip di triangoli

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>	
#include <GL/glu.h>	
#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <vcg/Point3.h>
#include <vcg/Matrix44.h>
#include <vector>
using namespace vcg;
using namespace std;

class myVertex
{
public :
	myVertex(Point3d &pp, Point4d &nn) {p=pp; n=Point3d(nn[0],nn[1],nn[2]);}
	Point3d p;
	Point3d n;
};

class myFace
{
public:
	myVertex *v[3];
	Point3d n;
};

class myMesh {
public:
	vector< myVertex > vert;
	vector< myFace > face;
	void ComputeFaceNormal();
	void DrawStrip() const ;
	bool empty() const {return vert.empty();}
	void clear() {vert.clear(); face.clear();}
};
// Hack!!
// Funziona solo perche' so come sono 
// stati creati i vertici in questo caso...
// La funzione di disegno della mesh dovrebbe essere generica...

void myMesh::DrawStrip() const
{
	glBegin(GL_TRIANGLE_STRIP);
	
	for(int i=0;i<vert.size();++i){
		glNormal(vert[i].n);
		glVertex(vert[i].p);
	}

	glEnd();
}



myMesh m;

void DrawBall(float radius)
{
	glPushAttrib(GL_LIGHTING_BIT);
	glPushMatrix();
	glShadeModel(GL_SMOOTH);
	static GLUquadricObj* sphere=0;
	if(!sphere) {
		sphere=gluNewQuadric();
		gluQuadricNormals(sphere,GLU_SMOOTH);
	}
	glColor3f(0,0,1);
	gluSphere(sphere,radius,16,8);
	glScalef(1.02,1.02,0.7);
	glColor3f(1,0,0);
	gluSphere(sphere,radius,16,8);
	glPopMatrix();
	glPopAttrib();
}

void DrawTranslatedRollingBall(float radius,float angle)
{
	const float MysteriousConstant = -3;
	glPushMatrix();
	glTranslatef(0,radius*1.5,0);
	glRotatef(angle*MysteriousConstant,1,0,0);
	DrawBall(radius);
  glPopMatrix();
}


void DrawMoebiusRollingBall(float r, int twist, float angle, float radius)
{
	float TwistPerDeg=(twist*180.0)/360.0f;

  	glPushMatrix();
		glRotatef(angle,0,1,0);
		glTranslatef(r,0,0);
		glRotatef(angle*TwistPerDeg,0,0,1);
		DrawTranslatedRollingBall(radius,angle);
		glPopMatrix();
}

void GenerateMoebiusRing(myMesh &m, int step, float r, int twist)
{
  m.clear();
	float angle=360.0f/step;
	float angletwist=(180.0f*twist)/step;
	for(int i=0;i<=step;++i)
	{
		Matrix44d mm;
		glPushMatrix();
		glLoadIdentity();
		glRotatef(i*angle,0,1,0);
		glTranslatef(r,0,0);
		glRotatef(i*angletwist,0,0,1);
		glGetModelViewMatrix(mm);
		m.vert.push_back(myVertex(mm*Point3d(-1,0,0),mm*Point4d(0,1,0,0)));
		m.vert.push_back(myVertex(mm*Point3d( 1,0,0),mm*Point4d(0,1,0,0)));
		glPopMatrix();
	}
}

void DrawGLScene()
{
	if(m.empty()) GenerateMoebiusRing(m,36,5,1 );
	static int LastT=clock();
	static float CurAngle=0;
  int CurT=clock();
	float DeltaT=(CurT-LastT)/1000.0;  // elapsed time in secs.
	if(DeltaT < 0.01) return;
	LastT=CurT;
	float AngularSpeed = 30;  // in degree/sec
	CurAngle+=AngularSpeed*DeltaT;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	
	float spec[4]={.4,.4,.4,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	float lightpos[4]={0,0,1,0};
	glLoadIdentity();
	glLightfv(GL_LIGHT0,GL_POSITION,lightpos);
	gluLookAt(2,5,12,0,0,0,0,1,0);
	glRotatef(-CurAngle,0,1,0);

  glColor3f(1,.4,.4);
	m.DrawStrip();	
	SDL_GL_SwapBuffers();
}

// Inizializzazione dello stato di OpenGL
void myGLInit()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);

}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	myGLInit();
}


int main(int argc, char **argv) 
{  
  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);   
  
	/* Create a 640x480 OpenGL screen */
  if ( SDL_SetVideoMode(640, 480, 0, SDL_OPENGL |SDL_RESIZABLE) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

	/* Loop, drawing and checking events */
	int done = 0;
	int ret;
  myGLReshapeFunc(640,480);  

	while ( ! done ) 
	{
    SDL_Event event;
	  ret=SDL_PollEvent(&event);
		if(ret)
			switch(event.type)
			{  
						case SDL_QUIT    : done = 1;  
															break ;
						case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
																				done = 1;
															break;
						case SDL_VIDEORESIZE	: SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL |SDL_RESIZABLE);
																		myGLReshapeFunc(event.resize.w,event.resize.h); 
																		break;
			}  
		else DrawGLScene();
  }
  SDL_Quit();
  return 1;
}
